/* -*- C++ -*- *************************************************************
 *
 *   rc_file.cpp, Mon Oct 27 2003
 *   Copyright (C) 2003 Alexei Soloviev <solovjev@cv.jinr.ru>
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rc_file.h"
#include "rc_sign.h"

#include <iterator>
#include <ctime>
#include <sstream>
#include <iomanip>
#include <algorithm>
#include <cctype>
using namespace std;

const string rc_file::_home = getenv("HOME") ? getenv("HOME") : "C:";

/** C-tor: using argument */
rc_file::rc_file ( const string & __name, ios::openmode __mode )
  : fstream (),
    _name(_home+"/."+__name),
    _mode(__mode)
{
  init();
}

#ifdef HAVE_CONFIG_H
#include <config.h>
/** C-tor: using definition from <config.h> */
rc_file::rc_file ( ios::openmode __mode )
  : fstream (),
    _name(_home+"/."+PACKAGE),
    _mode(__mode)
{
  init();
}
#endif

rc_file::~rc_file()
{
  close();
}

void rc_file::init()
{
  open(name().c_str(),ios::in);
  if ( is_open() ) {
    istream_iterator<string> cur ( *this ), eof;
    if ( good() ) { // stream can be damaged after this
      // date
      cur = ++find ( cur, eof, "on" );
      if ( count ( cur->begin(), cur->end(), '/' ) == 2 ) _date = *cur;
      // time
      cur = ++find ( cur, eof, "at" );
      if ( count ( cur->begin(), cur->end(), ':' ) == 2 ) _time = *cur;;
	} else {
	  clear();
	}
  }
  close();

  if ( _date.empty() || _time.empty() ) { // something wrong with header
    static time_t t;
    ::time(&t);
    struct tm * date = ::localtime(&t);
    // date
    ostringstream sdate;
    sdate.setf(ios::right,ios::adjustfield);
    sdate<<setfill('0')
	 <<date->tm_mday<<"/"
	 <<setw(2)<<date->tm_mon+1<<"/"
	 <<date->tm_year+1900;
    _date = sdate.str();
    // time
    ostringstream stime;
    stime.setf(ios::right,ios::adjustfield);
    stime<<setfill('0')
	 <<date->tm_hour<<":"
	 <<setw(2)<<date->tm_min<<":"
	 <<setw(2)<<date->tm_sec;
    _time = stime.str();
  }

  // header
  string _package = name().substr ( name().rfind ( "." )+1 );
  transform ( _package.begin(), _package.end(), _package.begin(), ::toupper );

  _header = 
    comment_sign() + "\n" +
    comment_sign() + "  " + _package + " configuration file" + "\n" +
    comment_sign() + "\n" +
    comment_sign() + "  This \"" + name() + "\" file is created" + "\n" + 
    comment_sign() + "  on " + date() + " at " + time() + "\n" +
    comment_sign() + "  You may edit it if you're careful!" + "\n" + 
    comment_sign() + "\n";

  // open
  open(name().c_str(),_mode);
}

/** Make rc-file backup */
void rc_file::backup()
{
  ofstream save ( (name()+".bak").c_str() );
  char buff[1024];
  while ( getline ( buff, 1024, '\n' ) ) save << buff << endl;
}

/** Remove rc-section from rc-file */
void rc_file::rm ( const string & rcsname )
{
  if ( _mode & ios::out ) {
    ifstream save ( (name()+".bak").c_str() );
    if ( save ) {
      char buff[1024];
      while ( save.getline ( buff, 1024, '\n' ) && buff != rcsname ) 
	*this << buff << endl;
      while ( save.getline ( buff, 1024, '\n' ) && buff != end_sign() );
      while ( save.getline ( buff, 1024, '\n' ) ) {
	*this << buff << endl;
      }
    }
  }
}

/** C-tor: using argument */
rc_file_i::rc_file_i ( const string & __name )
  : rc_file(__name,ios::in)
{}

#ifdef HAVE_CONFIG_H
#include <config.h>
/** C-tor: using definition from <config.h> */
rc_file_i::rc_file_i()
  : rc_file(ios::in)
{}
#endif

rc_file_i::~rc_file_i() {}

/** C-tor: using argument */
rc_file_o::rc_file_o ( const string & __name )
  : rc_file(__name,ios::out)
{
  if ( !good() ) clear();
  *this<<_header;
}

#ifdef HAVE_CONFIG_H
#include <config.h>
/** C-tor: using definition from <config.h> */
rc_file_o::rc_file_o()
  : rc_file(ios::out)
{
  if ( !good() ) clear();
  *this<<_header;
}
#endif

rc_file_o::~rc_file_o() {}

